package com.supervision.police.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supervision.common.utils.IPages;
import com.supervision.common.utils.ListUtils;
import com.supervision.common.utils.StringUtils;
import com.supervision.minio.domain.MinioFile;
import com.supervision.minio.mapper.MinioFileMapper;
import com.supervision.minio.service.MinioService;
import com.supervision.police.domain.ModelRecordType;
import com.supervision.police.domain.NoteRecordSplit;
import com.supervision.police.domain.NoteRecord;
import com.supervision.police.dto.NoteRecordDTO;
import com.supervision.police.dto.NoteRecordDetailDTO;
import com.supervision.police.mapper.ModelCaseMapper;
import com.supervision.police.mapper.NoteRecordSplitMapper;
import com.supervision.police.mapper.NoteRecordMapper;
import com.supervision.police.service.CaseStatusManageService;
import com.supervision.police.service.ModelRecordTypeService;
import com.supervision.police.service.NoteRecordSplitService;
import com.supervision.police.service.RecordSplitTypeService;
import com.supervision.springaidemo.dto.QARecordNodeDTO;
import com.supervision.springaidemo.util.RecordRegexUtil;
import com.supervision.springaidemo.util.WordReadUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class NoteRecordSplitServiceImpl extends ServiceImpl<NoteRecordSplitMapper, NoteRecordSplit> implements NoteRecordSplitService {

    private final NoteRecordMapper noteRecordMapper;

    private final MinioService minioService;

    private final ModelCaseMapper modelCaseMapper;

    private final MinioFileMapper minioFileMapper;

    @Autowired
    private ModelRecordTypeService modelRecordTypeService;

    @Autowired
    private RecordSplitTypeService recordSplitTypeService;

    private final CaseStatusManageService caseStatusManageService;


    @Override
    @Transactional(transactionManager = "dataSourceTransactionManager", rollbackFor = Exception.class)
    public String uploadRecords(NoteRecordDTO records) throws IOException {
        //修改案件的行为人、当事人
        /*if (StringUtils.isNotEmpty(records.getRole()) && !"嫌疑人".equals(records.getRole())) {
            ModelCase modelCase = modelCaseMapper.selectById(records.getCaseId());
            //行为人
            if ("行为人".equals(records.getRole()) && StringUtils.isEmpty(modelCase.getLawActor())) {
                modelCase.setLawActor(records.getName());
                modelCaseMapper.updateById(modelCase);
            } else if ("受害人".equals(records.getRole()) || "证人".equals(records.getRole())) {
                if (StringUtils.isEmpty(modelCase.getLawParty())) {
                    modelCase.setLawParty(records.getName());
                } else {
                    String[] split = modelCase.getLawParty().split(",");
                    if (!Arrays.asList(split).contains(records.getName())) {
                        modelCase.setLawParty(modelCase.getLawParty() + "," + records.getName());
                    }
                }
                modelCaseMapper.updateById(modelCase);
            }
        }*/
        int i;
        if (StringUtils.isEmpty(records.getId())) {
            i = noteRecordMapper.insert(records.toNoteRecord());
        } else {
            i = noteRecordMapper.updateById(records.toNoteRecord());
        }
        //所有对话类型
        List<ModelRecordType> allTypeList = modelRecordTypeService.lambdaQuery().list();
        if (i > 0 && CollUtil.isNotEmpty(records.getFileIdList())) {
            //拆分笔录
            for (String fileId : records.getFileIdList()) {
                MinioFile minioFile = minioService.getMinioFile(fileId);
                InputStream inputStream = null;
                try {
                    inputStream = minioService.getObjectInputStream(minioFile);
                } catch (Exception e) {
                    e.printStackTrace();
                    log.error("从minio中获取文件失败:{}", e.getMessage());
                    continue;
                }
                String context = WordReadUtil.readWord(inputStream);
                List<QARecordNodeDTO> qaList = RecordRegexUtil.recordRegex(context, records.getName());
                for (QARecordNodeDTO qa : qaList) {
                    try {
                        NoteRecordSplit noteRecord = new NoteRecordSplit();
                        noteRecord.setCaseId(records.getCaseId());
                        noteRecord.setNoteRecordsId(records.getId());
                        noteRecord.setNoteName(minioFile.getFilename());
                        noteRecord.setPersonName(records.getName());
                        noteRecord.setQuestion(qa.getQuestion());
                        noteRecord.setAnswer(qa.getAnswer());
                        noteRecord.setCreateTime(LocalDateTime.now());
                        this.save(noteRecord);
                        // 通过异步的形式提交分类
                        recordSplitTypeService.type(allTypeList, qa, noteRecord);
                    } catch (Exception e) {
                        log.error(e.getMessage(), e);
                    }
                }
            }
            caseStatusManageService.whenUploadRecord(records.getCaseId());
            return "保存成功";
        } else {
            return "保存笔录失败";
        }
    }

    @Override
    public Map<String, Object> queryRecords(NoteRecord noteRecords, Integer page, Integer size) {
        LambdaQueryWrapper<NoteRecord> wrapper = Wrappers.lambdaQuery(NoteRecord.class)
                .like(StringUtils.isNotEmpty(noteRecords.getName()), NoteRecord::getName, noteRecords.getName())
                .eq(NoteRecord::getCaseId, noteRecords.getCaseId())
                .eq(NoteRecord::getDataStatus, "1");
        List<NoteRecord> list = noteRecordMapper.selectList(wrapper);
        List<String> fileIdList = list.stream().map(NoteRecord::getFileIds).filter(StringUtils::isNotEmpty)
                .flatMap(s -> Arrays.stream(s.split(","))).distinct().toList();

        Map<String, MinioFile> fileMap = minioService.listMinioFile(fileIdList).stream().collect(Collectors.toMap(MinioFile::getId, Function.identity()));

        // LinkedHashMap 保障顺序
        Map<String, List<NoteRecordDetailDTO>> nameMap = list.stream().filter(item -> StringUtils.isNotBlank(item.getName())).map(item -> new NoteRecordDetailDTO(item, fileMap))
                .collect(Collectors.groupingBy(NoteRecordDetailDTO::getName, LinkedHashMap::new, Collectors.toList()));
        List<NoteRecordDetailDTO> res = new ArrayList<>();
        for (String name : nameMap.keySet()) {
            NoteRecordDetailDTO noteRecordDetailDTO = new NoteRecordDetailDTO();
            noteRecordDetailDTO.setName(name);
            noteRecordDetailDTO.setChildren(nameMap.get(name));
            noteRecordDetailDTO.setConfessionMaterial(CollUtil.size(noteRecordDetailDTO.getFileList()));
            res.add(noteRecordDetailDTO);
        }
        List<NoteRecordDetailDTO> pager = ListUtils.Pager(size, page, res);
        return IPages.buildDataMap(pager, res.size());
    }

    @Override
    public void delRecords(String id) {
        NoteRecord noteRecord = noteRecordMapper.selectById(id);
        noteRecord.setDataStatus(StringUtils.getUUID());
        noteRecordMapper.updateById(noteRecord);
        String fileIds = noteRecord.getFileIds();
        if (StringUtils.isNotEmpty(fileIds)) {
            //删除文件
            for (String fileId : fileIds.split(",")) {
                MinioFile minioFile = minioFileMapper.selectById(fileId);
                minioFile.setDataStatus(StringUtils.getUUID());
                minioFileMapper.updateById(minioFile);
                minioService.delFile(fileId);
            }
        }
        caseStatusManageService.whenDeleteRecord(noteRecord.getCaseId());
    }

}
